#!/usr/bin/env ruby

require 'artifactory'
require 'fileutils'
require 'json'
require 'omnibus'
require 'tempfile'

def self.env_or_empty(key)
  ENV[key] || ''
end

def self.env_or_raise(key)
  ENV[key] || raise("Required ENV variable `#{key}` is unset!")
end

# Values passed into the command sub-process by Jenkins
jenkins_home           = env_or_raise('JENKINS_HOME')
jenkins_workspace      = env_or_raise('WORKSPACE')
project_name           = env_or_raise('PROJECT_NAME')
omnibus_config_path    = File.join(jenkins_home, 'omnibus-publish.rb')

build_version   = ENV['OMNIBUS_BUILD_VERSION']
delivery_change = ENV['DELIVERY_CHANGE']

Dir.chdir(jenkins_workspace) do
  Omnibus.load_configuration(omnibus_config_path)

  client = Artifactory::Client.new(
    endpoint:       Omnibus::Config.artifactory_endpoint,
    username:       Omnibus::Config.artifactory_username,
    password:       Omnibus::Config.artifactory_password,
    ssl_pem_file:   Omnibus::Config.artifactory_ssl_pem_file,
    ssl_verify:     Omnibus::Config.artifactory_ssl_verify,
    proxy_username: Omnibus::Config.artifactory_proxy_username,
    proxy_password: Omnibus::Config.artifactory_proxy_password,
    proxy_address:  Omnibus::Config.artifactory_proxy_address,
    proxy_port:     Omnibus::Config.artifactory_proxy_port,
  )

  promotion_options = {
    comment: "Promoted to using opscode-ci's promotion script.",
    user: 'jenkins',
  }

  # Use the delivery change ID to get the build version when no build version has been specified.
  if build_version.nil?
    raise('Either `OMNIBUS_BUILD_VERSION` or `DELIVERY_CHANGE` must be set!') unless delivery_change

    build_artifacts = Artifactory::Resource::Artifact.property_search(
      'omnibus.project' => project_name,
      'delivery.change' => delivery_change,
      client: client,
    )

    # extract the build_number from one of the artifacts
    first_artifact = build_artifacts.first
    build_version = first_artifact.properties['build.number'].first # Artifactory returns property values as arrays
  end

  params = promotion_options.map { |k, v| "#{k}=#{v}" }
  promotion_url = "#{Omnibus::Config.artifactory_endpoint}/api/plugins/build/promote/current/#{project_name}/#{build_version}"
  promotion_url << "?params=#{params.join('|')}"
  promotion_url = URI.encode(promotion_url)

  begin
    client.post(promotion_url, nil)

    puts "Successfully promoted #{project_name}##{build_version} to omnibus-current-local"
  rescue ::Artifactory::Error::HTTPError => e
    raise <<-EOH.gsub(/^ {6}/, '')
      There was an error promoting #{project_name}##{build_version} to omnibus-current-local!

      Full error message from #{Omnibus::Config.artifactory_endpoint}:

      #{e.message}
     EOH
  end
end
